// src/hooks.ts
import { ref, unref, onMounted, onUnmounted, reactive } from "vue";
import {
  useQuery as useTanstackQuery,
  useMutation as useTanstackMutation,
  useQueryClient
} from "@tanstack/vue-query";
import { serialize } from "@wundergraph/sdk/internal";
var userQueryKey = "wg_user";
var createHooks = (client) => {
  const queryFetcher = async (query) => {
    const result = await client.query(query);
    if (result.error) {
      throw result.error;
    }
    return result.data;
  };
  const mutationFetcher = async (mutation) => {
    const result = await client.mutate(mutation);
    if (result.error) {
      throw result.error;
    }
    return result.data;
  };
  const queryKey = ({ operationName, input }) => {
    return [operationName, input];
  };
  const useQuery = (options) => {
    const { operationName, liveQuery, input, enabled, refetchOnWindowFocus, ...queryOptions } = options;
    const queryHash = serialize([operationName, input]);
    const result = useTanstackQuery({
      queryKey: queryKey({ operationName, input }),
      queryFn: ({ signal }) => queryFetcher({ operationName, input, abortSignal: signal }),
      ...queryOptions,
      enabled: liveQuery ? false : enabled,
      refetchOnWindowFocus: liveQuery ? false : refetchOnWindowFocus
    });
    const { isSubscribed } = useSubscribeTo(
      reactive({
        queryHash,
        operationName,
        input,
        liveQuery,
        enabled: options.enabled !== false && liveQuery,
        onSuccess: unref(options.onSuccess),
        onError: unref(options.onError)
      })
    );
    if (liveQuery) {
      return {
        ...result,
        isSubscribed
      };
    }
    return result;
  };
  const useMutation = (options) => {
    const { operationName, variables, ...mutationOptions } = options;
    const result = useTanstackMutation([operationName], (input) => mutationFetcher({ operationName, input }), {
      ...mutationOptions,
      variables
      //Bug with the vue-query types
    });
    return result;
  };
  const useAuth = () => {
    const queryClient = useQueryClient();
    return {
      login: (authProviderID, redirectURI) => client.login(authProviderID, redirectURI),
      logout: async (options) => {
        const result = await client.logout(options);
        queryClient.setQueryData([userQueryKey], null);
        return result;
      }
    };
  };
  const useUser = (options) => {
    const { revalidate, ...queryOptions } = options || {};
    return useTanstackQuery(
      [userQueryKey],
      ({ signal }) => client.fetchUser({
        revalidate,
        abortSignal: signal
      }),
      queryOptions
    );
  };
  const useFileUpload = (options) => {
    const { mutate, mutateAsync, ...mutation } = useTanstackMutation(
      ["uploadFiles"],
      async (input) => {
        const resp = await client.uploadFiles(input);
        return resp.fileKeys;
      },
      options
    );
    return {
      upload: mutate,
      uploadAsync: mutateAsync,
      ...mutation
    };
  };
  const subscribeTo = (options) => {
    const abort = new AbortController();
    const { onSuccess, onError, onResult, onAbort, ...subscription } = options;
    subscription.abortSignal = abort.signal;
    client.subscribe(subscription, onResult).catch(onError);
    return () => {
      onAbort == null ? void 0 : onAbort();
      abort.abort();
    };
  };
  const useSubscribeTo = (props) => {
    const client2 = useQueryClient();
    const { queryHash, operationName, input, enabled, liveQuery, subscribeOnce, resetOnMount, onSuccess, onError } = unref(props);
    const startedAt = ref(null);
    const isLoading = ref(false);
    const isSubscribed = ref(false);
    let unsubscribe = () => void 0;
    onMounted(() => {
      if (!startedAt.value && resetOnMount) {
        client2.removeQueries([operationName, input]);
      }
      if (enabled) {
        isLoading.value = true;
        isSubscribed.value = false;
        unsubscribe = subscribeTo({
          operationName,
          input,
          liveQuery: unref(liveQuery),
          subscribeOnce,
          onError(error) {
            isLoading.value = false;
            isSubscribed.value = false;
            onError == null ? void 0 : onError(error);
            startedAt.value = null;
          },
          onResult(result) {
            if (!startedAt.value) {
              isLoading.value = false;
              isSubscribed.value = true;
              onSuccess == null ? void 0 : onSuccess(result);
              startedAt.value = (/* @__PURE__ */ new Date()).getTime();
            }
            client2.setQueryData([operationName, input], () => {
              if (result.error)
                throw result.error;
              return result.data;
            });
          },
          onAbort() {
            isLoading.value = false;
            isSubscribed.value = false;
            startedAt.value = null;
          }
        });
      }
    });
    onUnmounted(() => {
      unsubscribe();
    });
    return { isLoading, isSubscribed };
  };
  const useSubscription = (options) => {
    const { enabled = true, operationName, input, subscribeOnce, onSuccess, onError } = options;
    const queryHash = serialize([operationName, input]);
    const subscription = useTanstackQuery({
      queryKey: [operationName, input],
      enabled: false
      // we update the cache async
    });
    const { isSubscribed } = useSubscribeTo(
      reactive({
        queryHash,
        operationName,
        input,
        subscribeOnce,
        enabled,
        onSuccess,
        onError
      })
    );
    return {
      ...subscription,
      isSubscribed
    };
  };
  return {
    useAuth,
    useFileUpload,
    useUser,
    useMutation,
    useQuery,
    useSubscription,
    queryKey
  };
};
export {
  createHooks
};
